//+------------------------------------------------------------------+
//|                                                      MA_MACD.mq4 |
//|                  a1ra@live.com  2010, MetaQuotes Software Corp. |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "a1ra@live.com  2010, MetaQuotes Software Corp."
#property link      ""

#include <stdlib.mqh>

extern string ExpertName = "MA_MACD";
extern int MagicNumber = 99;
extern bool BeepOnOpen = true;
extern bool BeepOnClose = true;

// Indicator setting
extern string Note1 = "MA cross setting";
extern int Fast_MA = 5;
extern int Fast_MA_Mode = 1;
extern int Fast_MA_Price = 0;
extern int Slow_MA = 15;
extern int Slow_MA_Mode = 2;
extern int Slow_MA_Price = 0;
extern string Note2 = "MACD setting";
extern int Fast_MACD = 12;
extern int Slow_MACD = 26;
extern int Signal_MACD = 9;


// Order management
extern bool CloseOnOpposite = false;
extern bool UseCloseSignal = false;

extern bool UseRiskManagement = false;
extern bool UseVariableRisk = false;
extern int OrderTypeBuy1 = OP_BUY;
extern int OrderTypeSell1 = OP_SELL;
extern double Lots1 = 0.1;
extern double Risk1 = 1;
extern int Slippage1 = 5;
extern int StopLoss1 = 25;
extern int TakeProfit1 = 50;
extern int Expiration1 = 0;

extern bool OpenSecondOrder = true;
extern int OrderTypeBuy2 = OP_BUY;
extern int OrderTypeSell2 = OP_SELL;
extern double Lots2 = 0.1;
extern double Risk2 = 2;
extern int Slippage2 = 5;
extern int StopLoss2 = 50;
extern int TakeProfit2 = 100;
extern int Expiration2 = 0;

extern int TrailingStop = 25;
extern bool UseTrailingStop = true;
extern bool UseCrawlingStop = false;

// Time and account protection management
extern bool LimitedHours = false;
extern int HoursFrom = 1;
extern int HoursTo = 23;
extern double TotalLossLimit = 500;
extern double TotalWinLimit = 900;
extern int MaxSpread = 6;
extern int OpenOrdersLimit = 4;
extern bool LimitOrdersPerDay = false;
extern int DayMaxOrderCount = 4;


// Local variables
string LastOrder = "None";
string Msg = "";
int Ticks = 0;
bool EnableNew = true;
double InitBalance = 0;
double PipValue = 0.0001;
datetime LastCrawling;
int CrawlingPause = 9;
int CrawlingStop = 5;
int CurrentDay = -1;
int DayCurrentOrderCount = 0;
bool VeryFirstOrders = true;


// Expert initialization
int init()
{
    LastCrawling = TimeCurrent();
    PipValue = Point;
    InitBalance = AccountBalance();
    VeryFirstOrders = true;
}

// Expert start
int start()
{
    if (Bars < 10)
    {
       Comment("Not enough bars");
       return (0);
    }

    if (CurrentDay != DayOfWeek())  // if day changed, reset day order count
    {
        DayCurrentOrderCount = 0;
        CurrentDay = DayOfWeek();
    }

    Ticks++;
    Msg = "";
    EnableNew = true;

    CheckAccountProtection();
    PrintToChart();

    if (UseCloseSignal)
        CloseOppositeOrders(CheckClosingSignal());

    int Signal = CheckOpeningSignal();
    if (Signal != 0)            // if there is a signal
    {
        if (CloseOnOpposite)    // if need to close already open positions
            CloseOppositeOrders(Signal);

        if (EnableNew)          // if new orders are allowed
        {
            double ask1 = Ask;
            double bid1 = Bid;
            if (OpenOrders(false, Signal, OrderTypeBuy1, OrderTypeSell1, ask1, bid1, Lots1, Risk1, StopLoss1, TakeProfit1, Slippage1, Expiration1) && OpenSecondOrder)
            {
                double ask2 = Ask;
                double bid2 = Bid;
                OpenOrders(true, Signal, OrderTypeBuy2, OrderTypeSell2, ask2, bid2, Lots2, Risk2, StopLoss2, TakeProfit2, Slippage2, Expiration2);
            }
            VeryFirstOrders = false;
        }
    }

    if (UseTrailingStop)
        Trail(TrailingStop);
    if (UseCrawlingStop && (LastCrawling - TimeCurrent() > CrawlingPause * 60))
    {
        LastCrawling = TimeCurrent();
        Trail(CrawlingStop);
    }

    return (0);
}

int CheckOpeningSignal()
{
    if (iMA(NULL, 0, Fast_MA, 0, Fast_MA_Mode, Fast_MA_Price, 0) > iMA(NULL, 0, Slow_MA, 0, Slow_MA_Mode, Slow_MA_Price, 0)
        && iMACD(NULL, 0, Fast_MACD, Slow_MACD, Signal_MACD, 0, 1, 0) > iMACD(NULL, 0, Fast_MACD, Slow_MACD, Signal_MACD, 0, 1, 1))
        return (1);
    if (iMA(NULL, 0, Fast_MA, 0, Fast_MA_Mode, Fast_MA_Price, 0) < iMA(NULL, 0, Slow_MA, 0, Slow_MA_Mode, Slow_MA_Price, 0)
        && iMACD(NULL, 0, Fast_MACD, Slow_MACD, Signal_MACD, 0, 1, 0) < iMACD(NULL, 0, Fast_MACD, Slow_MACD, Signal_MACD, 0, 1, 1))
        return (-1);

    return (0);
}

int CheckClosingSignal()
{
    
    

    return (0);
}

bool OpenOrders(bool SecondOrder, int Signal, int OrderTypeBuy0, int OrderTypeSell0, double Ask0, double Bid0, double Lots0, double Risk0, int StopLoss0, int TakeProfit0, int Slippage0, int Expiration0)
{
    double stoploss = 0;
    double takeprofit = 0;
    double price = 0;

    if (LimitOrdersPerDay && DayMaxOrderCount <= DayCurrentOrderCount)   // day limit reached for orders
        return (false);

    if (Signal == 1)
    {
        // prevents opening more orders if one of this type is already open
        if (!IsPosition(OrderTypeBuy0, Symbol(), MagicNumber) || SecondOrder)
        {
            LastCrawling = TimeCurrent();
            Lots0 = GetLotSize(Lots0, Risk0);
            stoploss = Ask - StopLoss0 * PipValue;
            takeprofit = Ask + TakeProfit0 * PipValue;
            price = Ask0;
            double expirebuy = TimeCurrent() + 60*Expiration0;    // wait for number of minutes for order to expire
            if (Expiration0 == 0)
                 expirebuy = 0;
            Print("BUY take profit = " + takeprofit, " stop loss = ", stoploss, " price = ", price);
            OrderSend(Symbol(), OrderTypeBuy0, Lots0, price, Slippage0, stoploss, takeprofit, ExpertName, MagicNumber, expirebuy, Blue);
            CheckError();
            LastOrder = "BUY";
            if (BeepOnOpen)
                PlaySound("ok.wav");
            DayCurrentOrderCount = DayCurrentOrderCount + 1;
            return (true);
        }
     }

     if (Signal == -1)
     {
         // prevents opening more orders if one of this type is already open
         if (!IsPosition(OrderTypeSell0, Symbol(), MagicNumber) || SecondOrder)
         {
             LastCrawling = TimeCurrent();
             Lots0 = GetLotSize(Lots0, Risk0);
             stoploss = Bid + StopLoss0 * PipValue;
             takeprofit = Bid - TakeProfit0 * PipValue;
             price = Bid0;
             double expiresell = TimeCurrent() + 60*Expiration0;    // wait for number of minutes for order to expire
             if (Expiration0 == 0)
                 expiresell = 0;
             Print("SELL take profit = " + takeprofit, " stop loss = ", stoploss, " price = ", price);
             OrderSend(Symbol(), OrderTypeSell0, Lots0, price, Slippage0, stoploss, takeprofit, ExpertName, MagicNumber, expiresell, Yellow);
             CheckError();
             LastOrder = "SELL";
             if (BeepOnOpen)
                 PlaySound("ok.wav");
             DayCurrentOrderCount = DayCurrentOrderCount + 1;
             return (true);
         }
      }
      return (false);
}

void CloseOppositeOrders(int Signal)
{
    // close any open positions if signal changes direction
    if (Signal == 1)
    {
        while (IsPosition(OP_SELL, Symbol(), MagicNumber))
        {
            OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 5, Red);   // close all prev. open SELLs
            if (BeepOnClose)
                PlaySound("ok.wav");
        }
        while (IsPosition(OP_SELLLIMIT, Symbol(), MagicNumber))
        {
            OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 5, Red);   // close all prev. open SELLs
            if (BeepOnClose)
                PlaySound("ok.wav");
        }

    }
    else if (Signal == -1)
    {
        while (IsPosition(OP_BUY, Symbol(), MagicNumber))
        {
            OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 5, Red);  // close all prev. open BUYs
            if (BeepOnClose)
                PlaySound("ok.wav");
        }
        while (IsPosition(OP_BUYLIMIT, Symbol(), MagicNumber))
        {
            OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), 5, Red);  // close all prev. open BUYs
            if (BeepOnClose)
                PlaySound("ok.wav");
        }
    }
}

void Trail(int TrailingStop0)
{
    for (int i=OrdersTotal()-1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
    {
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
        {
            if (OrderType() == OP_BUY && Bid - OrderOpenPrice() > TrailingStop0*PipValue)
            {
                if (OrderStopLoss() < Bid-(TrailingStop0+3)*PipValue)
                    OrderModify(OrderTicket(), OrderOpenPrice(), Bid-TrailingStop0*PipValue, Bid+(TakeProfit1 + TrailingStop0)*PipValue, OrderExpiration(), White);

            }
            if (OrderType() == OP_SELL && OrderOpenPrice() - Ask > TrailingStop0*PipValue)
            {
                if (OrderStopLoss() > Ask+(TrailingStop0+3)*PipValue)
                    OrderModify(OrderTicket(), OrderOpenPrice(), Ask+TrailingStop0*PipValue, Ask-(TakeProfit1 + TrailingStop0)*PipValue, OrderExpiration(), White);
            }
        }
    }
    else
        Print("OrderSelect() error - ", ErrorDescription(GetLastError()));
}

double GetLotSize(double Lots0, double Risk0)
{
    double lotvalue = Lots0;
    if (UseRiskManagement)
    {
        if (UseVariableRisk && VeryFirstOrders == false)
        {
            if (GetLastOrderProfit() < 0)   // decrease risk
                Risk0 = Risk0 - 0.1;
            else                            // increase risk
                Risk0 = Risk0 + 0.1;
        }

        //Determine micro/mini/standard broker
        double minilot = MarketInfo(Symbol(), MODE_MINLOT);
        int powerscount = 0;
        while (minilot < 1)
        {
            minilot = minilot * MathPow(10, powerscount);
            powerscount++;
        }

        lotvalue = NormalizeDouble(AccountFreeMargin() * Risk0 * 0.01 * MarketInfo(Symbol(), MODE_TICKSIZE), powerscount - 1);
    }
    if (lotvalue < MarketInfo(Symbol(), MODE_MINLOT))    // make sure lot is not smaller than allowed value
        lotvalue = MarketInfo(Symbol(), MODE_MINLOT);
    if (lotvalue > MarketInfo(Symbol(), MODE_MAXLOT))    // make sure lot is not greater than allowed value
        lotvalue = MarketInfo(Symbol(), MODE_MAXLOT);

    return (lotvalue);
}

void PrintToChart()
{
     string temp = ExpertName
         + "\n"
         + "Ticks: " + Ticks + " " + Msg
         + "\n"
         + " Last Order: " + LastOrder
         + "\n"
         + "------------------------------------------------"
         + "\n"
         + "ACCOUNT INFORMATION: "
         + "\n"
         + "Broker Company:     " + AccountCompany()+ "\n"
         + "Account Leverage:     " + DoubleToStr(AccountLeverage(), 0)+ "\n"
         + "Account Balance:     " + DoubleToStr(AccountBalance(), 2)+ "\n"
         + "Account Equity:     " + DoubleToStr(AccountEquity(), 2)+ "\n"
         + "Free Margin:     " + DoubleToStr(AccountFreeMargin(), 2)+ "\n"
         + "Used Margin:     " + DoubleToStr(AccountMargin(), 2)+ "\n"
         
         + "------------------------------------------------";
    Comment(temp);
}

// provides crude tools to protect account against losses, in most cases simply disables
// openning of new orders and alerts the user.
void CheckAccountProtection()
{
    Msg = "";
    if (AccountFreeMargin() < (1000*Lots1))
    {
        Msg = "--> WARN: NO MONEY.";
        EnableNew = false;
        return (0);
    }

    // Loss protection will wait until this expert is restarted
    if (AccountBalance() - InitBalance < TotalLossLimit * (-1))
    {
        Msg = "--> WARN: Total Loss Protection. No new orders will be open.";
        EnableNew = false;
        return (0);
    }

    // Win protection will wait until this expert is restarted
    if (AccountBalance() - InitBalance > TotalWinLimit)
    {
        Msg = "--> WARN: Total Win Protection. No new orders will be open.";
        EnableNew = false;
        return (0);
    }

    if (LimitedHours && IsTradeTime() == false)
    {
        Msg = "--> WARN: Not trading hours. No new orders will be open.";
        EnableNew = false;
        return (0);
    }

    // spread protection
    int spread = MarketInfo(Symbol(), MODE_SPREAD);
    if (spread > MaxSpread)
    {
        Msg = "--> WARN: Spread is " + spread + ". No new orders will be open.";
        EnableNew = false;
        return (0);
    }

    if (GetOpenOrderCount(Symbol(), MagicNumber) > OpenOrdersLimit)
    {
        Msg = "--> WARN: Open orders limit reached " + OpenOrdersLimit + ". No new orders will be open.";
        EnableNew = false;
        return (0);
    }

    if (LimitOrdersPerDay && DayMaxOrderCount <= DayCurrentOrderCount)
    {
        Msg = "--> WARN: Number of orders per day limit reached " + DayMaxOrderCount + ". No new orders will be open.";
        EnableNew = false;
        return (0);
    }

    

    

}

// -- Utility functions

void CheckError()
{
    int err = GetLastError();
    if (err != 0)
        Print("error(",err,"): ", ErrorDescription(err));
}

bool IsPosition(int type, string symbol, int MagicNumber)
{
    for (int i=OrdersTotal()-1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
    {
        if (OrderType() == type)
        if (OrderSymbol() == symbol)
        if (OrderMagicNumber() == MagicNumber)
            return (true);
    }
    else
        Print("OrderSelect() error - ", ErrorDescription(GetLastError()));
    return (false);
}

int GetOpenOrderCount(string symbol, int MagicNumber)
{
    int count = 0;
    for (int i=OrdersTotal()-1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
    {
        if (OrderSymbol() == symbol)
        if (OrderMagicNumber() == MagicNumber)
            count++;
    }
    else
        Print("OrderSelect() error - ", ErrorDescription(GetLastError()));
    return (count);
}

bool IsTradeTime()
{
   int datetime800 = TimeLocal();
   int hour0 = TimeHour(datetime800);

   if (hour0 < HoursFrom || hour0 >= HoursTo)
        return (false);
   return (true);
}

double GetLastOrderProfit()
{
    double profit = 0;
    datetime lastCloseTime = 0;
    int cnt = OrdersHistoryTotal();
    for (int i=0; i < cnt; i++)
    {
        if (!OrderSelect(i, SELECT_BY_POS, MODE_HISTORY)) continue;
        if (OrderSymbol() != Symbol()) continue;
        if (OrderMagicNumber() != MagicNumber) continue;

        if (lastCloseTime < OrderCloseTime())   // find most recent order
        {
            lastCloseTime = OrderCloseTime();
            profit = OrderProfit();
        }
    }
    return (profit);
}


